import pygame
import pygame.freetype

class Shell:
    lines = ["~/INSERT_NAME > HELP"]
    cursor = [0, 0]
    scroll = [0, 0]
    font = None
    blink_timer = 0
    surf = None
    button_done = None
    done_rect = None
    button_help = None
    help_rect = None
    help_open = False

    def button_surf (self, text):
        surf = pygame.Surface ((80, 24))
        pygame.draw.rect (surf, (0, 255, 0), pygame.Rect (0, 0, 80, 24), 1)
        self.font.render_to (surf, (40 - 3 * len (text), 8), text, (0, 255, 0))
        return surf
        
    def __init__(self):
        self.lines = ["~/INSERT_NAME > HELP"]
        self.cursor = [0, 0]
        self.scroll = [0, 0]
        self.blink_timer = 0
        self.help_open = False
        self.surf = pygame.Surface ((480, 264))
        try:
            self.font = pygame.freetype.Font("font/gohufont-11.bdf")
        except:
            raise RuntimeError("Could not load fonts. Please download again")
        self.button_help = self.button_surf ("HELP")
        self.help_rect = None
        self.button_done = self.button_surf ("DONE")
        self.done_rect = None

    def set_text (self, text):
        self.cursor = [0, 0]
        self.lines = text.split ("\n")

    def update (self):
        self.blink_timer = (self.blink_timer + 1) % 50

    def add_to_input (self, letter):
        if letter == '\n':
            cur_line = self.lines[self.cursor[0]]
            self.lines[self.cursor[0]] = cur_line[:self.cursor[1]]
            self.lines.insert (self.cursor[0] + 1, cur_line[self.cursor[1]:])
            self.scroll[1] = 0
            self.cursor[1] = 0
            self.cursor[0] += 1
            if self.cursor[0] - self.scroll[0] > 23:
                self.scroll[0] += 12
        else:
            self.lines[self.cursor[0]] = self.lines[self.cursor[0]][:self.cursor[1]] + letter + self.lines[self.cursor[0]][self.cursor[1]:]
            self.cursor[1] += 1
            if self.cursor[1] - self.scroll[1] > 79:
                self.scroll[1] += 40

    def backspace (self):
        if self.cursor[1] == 0:
            if self.cursor[0] > 0:
                self.cursor[1] = len (self.lines[self.cursor[0] - 1])
                self.scroll[1] = 0
                self.lines[self.cursor[0] - 1] += self.lines[self.cursor[0]]
                while self.cursor[1] - self.scroll[1] > 79:
                    self.scroll[1] += 40
                self.lines.pop(self.cursor[0])
                self.cursor[0] -= 1
                if self.cursor[0] < self.scroll[0]:
                    self.scroll[0] -= 12
        else:
            self.lines[self.cursor[0]] = self.lines[self.cursor[0]][:self.cursor[1] - 1] + self.lines[self.cursor[0]][self.cursor[1]:]
            self.cursor[1] -= 1
            if self.cursor[1] < self.scroll[1]:
                self.scroll[1] -= 40

    def move (self, key):
        def adjust_col_cursor ():
            if self.cursor[1] > len (self.lines[self.cursor[0]]):
                self.cursor[1] = len (self.lines[self.cursor[0]])
            self.scroll[1] = 0
            while self.cursor[1] - self.scroll[1] > 79:
                self.scroll[1] += 12
        
        if key == pygame.K_UP and self.cursor[0] > 0:
            self.cursor[0] -= 1
            if self.cursor[0] < self.scroll[0]:
                self.scroll[0] -= 12
            adjust_col_cursor ()
        elif key == pygame.K_DOWN and self.cursor[0] < len (self.lines) - 1:
            self.cursor[0] += 1
            if self.cursor[0] - self.scroll[0] > 23:
                self.scroll[0] += 12
            adjust_col_cursor ()
        elif key == pygame.K_LEFT and self.cursor[1] > 0:
            self.cursor[1] -= 1
            if self.cursor[1] < self.scroll[1]:
                self.scroll[1] -= 40
        elif key == pygame.K_RIGHT and self.cursor[1] < len (self.lines[self.cursor[0]]):
            self.cursor[1] += 1
            if self.cursor[1] - self.scroll[1] > 79:
                self.scroll[1] += 40

    def input (self, key):
        self.blink_timer = 0
        if key == pygame.K_BACKSPACE:
            self.backspace ()
        elif key == pygame.K_SPACE:
            self.add_to_input (' ')
        elif key == pygame.K_RETURN:
            self.add_to_input ('\n')
        elif (key == pygame.K_UP or key == pygame.K_DOWN or
              key == pygame.K_LEFT or key == pygame.K_RIGHT):
            self.move (key)
        elif key in range (ord ('a'), ord ('z') + 1):
            self.add_to_input (chr (key - 32))

    def mouse_down (self, pos):
        if self.help_rect != None and self.done_rect != None:
            if self.help_rect.collidepoint (pos):
                self.help_open = True
            if self.done_rect.collidepoint (pos):
                return True
        return False
    
    def get_surf (self):
        self.surf.fill ((0, 0, 0))
        for i in range(24):
            if i + self.scroll[0] >= len (self.lines):
                break
            if i + self.scroll[0] == self.cursor[0]:
                text = self.lines[self.cursor[0]][self.scroll[1]:]
            else:
                text = self.lines[self.scroll[0] + i]
            self.font.render_to (self.surf, (0, i*11), text, (0, 255, 0))
        if (self.blink_timer < 25):
            cursor = pygame.Rect ((self.cursor[1] - self.scroll[1]) * 6, (self.cursor[0] - self.scroll[0]) * 11, 5, 9)
            pygame.draw.rect (self.surf, (0, 255, 0), cursor)
        return self.surf

    def draw (self, screen):
        screen_size = screen.get_size ()
        scale_factor = 2
        if screen_size[0] < 480 * 2  or screen_size[1] < 264 * 2 + 120:
            scale_factor = 1
        display_size = (480*scale_factor, 264*scale_factor)
        button_size = (80*scale_factor, 24*scale_factor)
        surf = pygame.transform.scale (self.get_surf (), display_size)

        pygame.draw.rect (screen, (0, 255, 0), pygame.Rect ((screen_size[0] - (480 + 8) * scale_factor) // 2, (screen_size[1] - (60 + 264 + 8) * scale_factor) // 2, (480 + 8) * scale_factor, (264 + 8) * scale_factor), scale_factor)
        screen.blit (surf, pygame.Rect ((screen_size[0] - display_size[0]) // 2, (screen_size[1] - 60 * scale_factor - display_size[1]) // 2, display_size[0], display_size[1]))

        self.done_rect = pygame.Rect ((screen_size[0] + display_size[0]) // 2 - button_size[0], screen_size[1] - 75 * scale_factor, button_size[0], button_size[1])
        self.help_rect = pygame.Rect ((screen_size[0] + display_size[0]) // 2 - 2 * button_size[0] - 12*scale_factor, screen_size[1] - 75 * scale_factor, button_size[0], button_size[1])
        screen.blit (pygame.transform.scale (self.button_done, button_size), self.done_rect)
        screen.blit (pygame.transform.scale (self.button_help, button_size), self.help_rect)

