import pygame
import pygame.freetype

class Dialog_box:
    text = ""
    lines = []
    cursor = [0, 0]
    font = None
    character_sprite = None
    window_size = (0, 0)
    sprite_size = (0, 0)
    pausing = False
    finished = True
    max_lines = 0
    scroll_indicator = None

    def cut (self):
        sentences = self.text.split ("\n")
        total = []
        for s in sentences:
            words = s.split ()
            result = []
            while len (words) > 0:
                size = 0
                n = 1
                while size < self.window_size[0] - self.sprite_size[0] - 40 and n <= len (words):
                    n += 1
                    rect = self.font.get_rect (str.join (" ", words[:n]))
                    size = rect[2]
                n = max (n - 1, 2)
                result.append (str.join (" ", words[:n]))
                words = words[n:]
            total += result
        self.lines = total

    def compute_max_lines (self):
        height = self.window_size[1] // 4 - 20 #10 is margins
        self.max_lines = height // self.font.get_sized_height ()
    
    def __init__(self, font, window_size, character_sprite):
        self.font = font
        self.font.origin = True
        self.window_size = window_size
        self.text = ""
        self.cursor = [0, 0]
        self.pausing = False
        self.finished = True
        self.scroll_indicator = pygame.image.load ("img/scroll_indicator.png")

        sprite_size = character_sprite.get_size ()
        sprite_width = min (sprite_size[0], window_size[0] / 4.0)
        sprite_height = float (sprite_size[1]) * sprite_width / sprite_size[0]

        self.sprite_size = (int (sprite_width), int (sprite_height))
        self.character_sprite = pygame.transform.smoothscale (character_sprite, self.sprite_size)
        self.cut () #emo
        self.compute_max_lines ()

    def set_text (self, text):
        self.text = text
        self.cursor = [0, 0]
        self.pausing = False
        self.finished = False
        self.cut ()

    def update (self):
        if (not self.pausing) and (not self.finished) :
            self.cursor[1] += 1
            if self.cursor[1] > len (self.lines[self.cursor[0]]):
                if self.cursor[0] == self.max_lines - 1 or self.cursor[0] == len (self.lines) - 1:
                    self.pausing = True
                else:
                    self.cursor[1] = 0
                    self.cursor[0] += 1

    def enter (self):
        if not self.finished:
            if not self.pausing:
                while not self.pausing:
                    self.update ()
            elif self.cursor[0] == len (self.lines) - 1:
                self.finished = True
            else:
                self.pausing = False
                self.lines = self.lines[self.cursor[0] + 1:]
                self.cursor = [0, 0]

    def draw (self, screen):
        if not self.finished:
            rect = pygame.Rect (0, (self.window_size[1] * 3) // 4, self.window_size[0], self.window_size[1] // 4)
            pygame.draw.rect (screen, (174, 180, 248), rect)
            for i in range (self.cursor[0] + 1):
                text = self.lines[i]
                if i == self.cursor[0]:
                    text = text[:self.cursor[1]]
                self.font.render_to (screen, (self.sprite_size[0] + 10, (self.window_size[1] * 3) // 4 + 5 + (i + 1) * self.font.get_sized_height ()), text, (0, 0, 0))
                screen.blit (self.character_sprite, pygame.Rect (0, self.window_size[1] - self.sprite_size[1], self.sprite_size[0], self.sprite_size[1]))
            if self.pausing:
                screen.blit (self.scroll_indicator, pygame.Rect (self.window_size[0] - 26, self.window_size[1] - 20, 16, 10))


# disgusting copypasta
# sorry
class Monolog_box:
    text = ""
    lines = []
    cursor = [0, 0]
    font = None
    window_size = (0, 0)
    pausing = False
    finished = True
    visible = False
    max_lines = 0
    scroll_indicator = None

    def cut (self):
        sentences = self.text.split ("\n")
        total = []
        for s in sentences:
            words = s.split (" ")
            result = []
            while len (words) > 0:
                size = 0
                n = 1
                while size < self.window_size[0] - 100 and n <= len (words):
                    n += 1
                    rect = self.font.get_rect (str.join (" ", words[:n]))
                    size = rect[2]
                n = max (n - 1, 2)
                result.append (str.join (" ", words[:n]))
                words = words[n:]
            total += result
        self.lines = total

    def compute_max_lines (self):
        height = self.window_size[1] // 2
        self.max_lines = height // self.font.get_sized_height ()
    
    def __init__(self, font, window_size):
        self.font = font
        self.font.origin = True
        self.window_size = window_size
        self.text = ""
        self.cursor = [0, 0]
        self.pausing = False
        self.finished = True
        self.visible = True
        self.scroll_indicator = pygame.image.load ("img/scroll_indicator_white.png")
        self.cut ()
        self.compute_max_lines ()

    def set_text (self, text):
        self.text = text
        self.cursor = [0, 0]
        self.pausing = False
        self.finished = False
        self.cut ()

    def update (self):
        if (not self.pausing) and (not self.finished) :
            self.cursor[1] += 1
            if self.cursor[1] > len (self.lines[self.cursor[0]]):
                if self.cursor[0] == self.max_lines - 1 or self.cursor[0] == len (self.lines) - 1:
                    self.pausing = True
                else:
                    self.cursor[1] = 0
                    self.cursor[0] += 1

    def enter (self):
        if not self.finished:
            if not self.pausing:
                while not self.pausing:
                    self.update ()
            elif self.cursor[0] == len (self.lines) - 1:
                self.finished = True
            else:
                self.pausing = False
                self.lines = self.lines[self.cursor[0] + 1:]
                self.cursor = [0, 0]

    def draw (self, screen):
        if self.visible:
            rect = pygame.Rect (0, 0, 0, 0)
            for i in range (self.cursor[0] + 1):
                text = self.lines[i]
                if i == self.cursor[0]:
                    text = text[:self.cursor[1]]
                rect = self.font.get_rect (text)
                self.font.render_to (screen, ((self.window_size[0] - rect[2])//2, self.window_size[1] // 4 + (i + 1) * self.font.get_sized_height ()), text, (255, 255, 255))
            if self.pausing:
                screen.blit (self.scroll_indicator, pygame.Rect ((self.window_size[0] + rect[2]) // 2 + 20, self.window_size[1] // 4 + (self.cursor[0] + 1) * self.font.get_sized_height (), 16, 10))

