import sys
import settings_manager
import command_prompt
import tiny_parser as parser
import simulator
import dialogs
import scenario
import pygame
import pygame.freetype

network_total_time = 1500

class Game:
    state = 0
    level = 0
    level_step = 0
    dialog = False
    base_font = None
    computer_font = None
    settings = None
    screen = None
    shell = None
    mc_sprite = None
    mc_dialog_box = None
    monologue = None
    chat = None
    busy = False
    network = None
    network_surf = None
    network_time = 0
    network_last_scores = None
    shown_network_tutorial = False
    blobs = []
    
    def __init__(self):
        self.state = 0
        self.level = 0
        self.level_step = -1
        self.dialog = False
        self.busy = False

        self.network = None
        self.network_time = 0
        self.network_surf = None
        self.network_last_scores = None
        self.shown_network_tutorial = False

        self.settings = settings_manager.load_settings ()
        self.screen = pygame.display.set_mode(self.settings["window_size"])
        self.shell = command_prompt.Shell ()

        self.base_font = pygame.freetype.Font ("font/ProzaLibre-Regular.ttf", self.settings["font_size"])
        self.computer_font = pygame.freetype.Font ("font/pokemon_fire_red.ttf", int (1.5 * self.settings["font_size"]))
        self.mc_sprite = pygame.image.load ("img/mc.png")
        self.mc_dialog_box = dialogs.Dialog_box (self.base_font, self.settings["window_size"], self.mc_sprite)
        self.monologue = dialogs.Monolog_box (self.base_font, self.settings["window_size"])
        self.chat = dialogs.Monolog_box (self.computer_font, self.settings["window_size"])
        self.next_step ()

        self.blobs = [None, None, None]
        self.blobs[0] = pygame.image.load ("img/01.png")
        self.blobs[1] = pygame.image.load ("img/yukkuri.png")
        self.blobs[2] = pygame.image.load ("img/nsa.png")

    def update (self):
        if self.dialog:
            self.mc_dialog_box.update ()
        elif self.state == 0:
            self.monologue.update ()
            self.chat.update ()
        elif self.state == 2 and self.network != None:
            if self.network_time < network_total_time:
                if self.network_time % 10 == 0:
                    self.network.simulate_global_battle ()
                    if self.network.possessions == self.network_last_scores:
                        self.network_time = network_total_time
                        self.results ()
                    self.network_last_scores = self.network.possessions.copy ()
                    surf_size = min (self.screen.get_size ()[0] - 300, self.screen.get_size ()[1])
                    self.network_surf = self.network.draw ((surf_size, surf_size))
                self.network_time += 1
            else:
                self.results ()
        if self.state == 1:
            self.shell.update ()

    def enter (self):
        if self.dialog:
            self.mc_dialog_box.enter ()
            if self.mc_dialog_box.finished:
                self.dialog = False
                if not self.busy:
                    self.next_step ()
                if self.busy == 2:
                    self.state = 1
        elif self.state == 0:
            self.monologue.enter ()
            self.chat.enter ()
            if self.monologue.finished and self.chat.finished and (not self.busy):
                self.next_step ()

    def setup_network (self):
        automata = scenario.automata[self.level - 1].copy ()
        automata[1] = self.automaton
        self.network_time = 0
        self.network = simulator.Network (automata, 25, 25)
        self.network.implant (1, 0)
        for i in range (20):
            self.network.extend_territory (1, 0)
        for j in range (2, len (automata)):
            self.network.implant (j, 0)
            for i in range (100):
                self.network.extend_territory (j, 0)
        surf_size = min (self.screen.get_size ()[0] - 300, self.screen.get_size ()[1])
        self.network_surf = self.network.draw ((surf_size, surf_size))

    def results (self):
        if self.network.possessions[1] >= (3 * self.network.total_size) // 4:
            self.dialog = True
            self.busy = False
            self.mc_dialog_box.set_text (scenario.won)
        else:
            self.dialog = True
            self.busy = 2 #ugly fix
            self.mc_dialog_box.set_text (scenario.lost)
                
    def compile_code (self):
        try:
            self.automaton = parser.parse (self.shell.lines)
            self.state = 2
            self.setup_network ()
            if not self.shown_network_tutorial:
                self.busy = True
                self.dialog = True
                self.mc_dialog_box.set_text (scenario.networking_tutorial)
                self.shown_network_tutorial = True
        except RuntimeError as compile_error:
            self.dialog = True
            self.busy = True
            self.mc_dialog_box.set_text (str (compile_error))
        
    def event (self, event):
        if event.type == pygame.KEYDOWN:
            if self.dialog or self.state == 0:
                if event.key == pygame.K_SPACE or event.key == pygame.K_RETURN:
                    self.enter ()
            elif self.state == 1:
                self.shell.input (event.key)

        elif event.type == pygame.MOUSEBUTTONDOWN:
            if self.dialog or self.state == 0:
                self.enter ()
            elif self.state == 1:
                if (self.shell.mouse_down (event.pos)):
                    self.compile_code ()

    def draw_network (self):
        if self.network_surf != None:
            surf_size = self.network_surf.get_size ()
            screen_size = self.screen.get_size ()
            self.screen.blit (self.network_surf, pygame.Rect (0, (screen_size[1] - surf_size[1]) // 2, surf_size[0], surf_size[1]))
            scores = self.network.possessions
            self.screen.blit (self.blobs[1], pygame.Rect ((surf_size[0] + screen_size[0]) // 2 - 100, screen_size[1] // 4 - 40, 100, 87))
            self.base_font.render_to (self.screen, ((surf_size[0] + screen_size[0]) // 2 + 50, screen_size[1] // 4 + 20), str (scores[1]), (255, 255, 255))
            self.screen.blit (self.blobs[0], pygame.Rect ((surf_size[0] + screen_size[0]) // 2 - 100, screen_size[1] // 2 - 40, 100, 87))
            self.base_font.render_to (self.screen, ((surf_size[0] + screen_size[0]) // 2 + 50, screen_size[1] // 2 + 20), str (scores[0]), (255, 255, 255))
            if len (scores) > 2:
                self.screen.blit (self.blobs[2], pygame.Rect ((surf_size[0] + screen_size[0]) // 2 - 100, screen_size[1] * 3 // 4 - 40, 100, 87))
                self.base_font.render_to (self.screen, ((surf_size[0] + screen_size[0]) // 2 + 50, screen_size[1] * 3 // 4 + 20), str (scores[2]), (255, 255, 255))

    def draw (self):
        self.screen.fill ((0, 0, 0))
        if self.state == 0:
            self.monologue.draw (self.screen)
            self.chat.draw (self.screen)
        elif self.state == 1:
            self.shell.draw (self.screen)
        elif self.state == 2:
            self.draw_network ()
        if self.dialog:
            self.mc_dialog_box.draw (self.screen)

    def next_step (self):
        self.level_step += 1
        if self.level_step >= len (scenario.story[self.level]):
            self.level += 1
            if self.level >= len (scenario.story):
                self.state = 3
            else:
                self.level_step = -1
                self.state = 1
        else:
            new_step = scenario.story[self.level][self.level_step]
            if new_step[0] == 0:
                self.state = 0
                self.chat.visible = False
                self.monologue.visible = True
                self.monologue.set_text (new_step[2])
            if new_step[0] == 2:
                self.state = 0
                self.monologue.visible = False
                self.chat.visible = True
                self.chat.set_text (new_step[2])
            if new_step[0] == 1:
                self.dialog = True
                self.busy = False
                self.mc_dialog_box.set_text (new_step[2])
            if new_step[0] == 3:
                self.state = 1
                self.next_step ()
            if new_step[0] == 4:
                self.shell.set_text (new_step[2])
                self.next_step ()
            

def main ():
    pygame.init ()
    pygame.freetype.init ()
    pygame.key.set_repeat (300, 100)
    
    game = Game ()

    while True:
        start_time = pygame.time.get_ticks ()

        game.update ()

        for event in pygame.event.get ():
            if event.type == pygame.QUIT:
                sys.exit (0)
            else:
                game.event (event)
            
        game.draw ()
        pygame.display.flip ()

        ellapsed_time = pygame.time.get_ticks () - start_time
        if ellapsed_time < 33:
            pygame.time.wait (33 - ellapsed_time)
            
main ()
