import math
import random
import pygame

player_colors = [ (0, 255, 0), (233, 147, 255), (255, 0, 0), (0, 255, 0), (255, 0, 0) ]

def random_permutation (lst):
    n = len (lst)
    for i in range (n - 1):
        new = random.randint(i, n - 1)
        lst[i], lst[new] = lst[new], lst[i]

class Network:
    servers = []
    width = 32
    height = 32
    total_size = 1024
    margin = 5
    nb_player = 1
    possessions = [1024]
    automata = [{'START': (0, 'START', 'START')}]
    
    # server1 and server2 should be player IDs
    def simulate_battle  (self, server1, server2):
        nb_fights = random.randint (10, 30)
        scores = [0, 0]
        state1, state2 = "START", "START"
        results = [[(20, 20), (120, 0 ), (20, 40)],
                   [(0 , 120), (70, 70), (70, 40)],
                   [(40, 20), (40, 70), (40, 40)]]
        
        for i in range (nb_fights):
            behaviours = (self.automata[server1][state1][0],
                          self.automata[server2][state2][0])
            scores[0] += results[behaviours[0]][behaviours[1]][0]
            scores[1] += results[behaviours[0]][behaviours[1]][1]
            state1 = self.automata[server1][state1][1 + min (behaviours[1], 1)]
            state2 = self.automata[server2][state2][1 + min (behaviours[0], 1)]

        return [float (scores[0]) / nb_fights, float (scores[1]) / nb_fights]

    def execute_transactions (self, transactions):
        for i in range (self.nb_players):
            for j in range (i + 1, self.nb_players):
                quantity = transactions[i][j] - transactions[j][i]
                if quantity > 0:
                    for k in range (quantity):
                        self.extend_territory (j, i)
                else:
                    for k in range (-quantity):
                        self.extend_territory (i, j)


    def simulate_global_battle (self):
        lst = [i for i in range (self.total_size)]
        player_scores = [0 for i in range (self.nb_players)]
        random_permutation (lst)
        for i in range (len (lst) // 2):
            scores = self.simulate_battle (self.servers[lst[2*i]][2], self.servers[lst[2*i + 1]][2])
            player_scores[self.servers[lst[2*i]][2]] += scores[0]
            player_scores[self.servers[lst[2*i + 1]][2]] += scores[1]

        total_score = sum (player_scores)
        for i in range (self.nb_players):
            player_scores[i] = float (player_scores[i]) / total_score
            
        def get_transaction (i, j):
            return int (self.possessions[i] * player_scores[j] / 2)
        transactions = [[get_transaction (i, j) for j in range (self.nb_players)] for i in range (self.nb_players)]
        self.execute_transactions (transactions)

    # returns a graph of servers, in the format [ [position of server 1, neighbours, occupant], ... ]
    def __init__(self, automata, width = 32, height = 32, margin = 5):
        self.width = width
        self.height = height
        self.total_size = width * height
        self.nb_players = len (automata)
        self.automata = automata
        self.possessions = [0 for i in range (self.nb_players)]
        self.possessions[0] = width * height
        self.margin = margin
        def index (x, y):
            return y * self.width + x
        def in_range (x, y):
            return 0 <= x and x < self.width and 0 <= y and y < self.height
        self.servers = [[(16, 16), set (), 0] for i in range (self.total_size)]
    
        for i in range (self.width):
            for j in range (self.height):
                neighbours = self.servers[index (i, j)][1]
                for k1 in range (-2, 3):
                    for k2 in range (-2, 3):
                        if (k1 != 0 or k2 != 0) and in_range (i + k1, j + k2):
                            proba = 0.15 / ((abs (k1) + abs (k2)))
                            if random.random () <= proba:
                                neighbours.add (index (i + k1, j + k2))
                                self.servers[index (i + k1, j + k2)][1].add (index (i, j))
                if len (neighbours) == 0:
                    default = i > 0 and index (i - 1, j) or index (i + 1, j)
                    neighbours = {default}
                position = (i + random.random (), j + random.random ())
                self.servers[index (i, j)] = [position, neighbours, 0]

    def get_base_server (self, player):
        result = None
        for i in range (self.total_size):
            if self.servers[i][2] == player:
                result = i
                break
        return result

    def extend_territory (self, player, defeated, base = None):
        if base == None:
            base = self.get_base_server (player)
            test = self.get_base_server (defeated)
        if base != None and test != None:
            stack = list (self.servers[base][1])
            explored = {base:True}
            success = False
            while len (stack) != 0:
                cur = stack.pop ()
                explored[cur] = True
                if self.servers[cur][2] == defeated:
                    self.servers[cur][2] = player
                    self.possessions[player] += 1
                    self.possessions[defeated] -= 1
                    success = True
                    break
                else:
                    neighbours = list(self.servers[cur][1])
                    random_permutation (neighbours)
                    for n in neighbours:
                        if not (n in explored):
                            stack.append(n)
            if not success:
                self.servers[test][2] = player
                self.possessions[player] += 1
                self.possessions[defeated] -= 1

    def implant (self, player, defeated):
        serv = self.get_base_server (defeated)
        if serv != None:
            self.servers[serv][2] = player
            self.possessions[player] += 1
            self.possessions[defeated] -= 1

    def draw (self, size):
        def surf_coord (graph_coord):
            return (int (graph_coord[0] * (size[0] - 2*self.margin) / (self.width) + self.margin),
                    int (graph_coord[1] * (size[1] - 2*self.margin) / (self.height) + self.margin))
        surf = pygame.Surface (size)
        for serv in self.servers:
            for neighbour in serv[1]:
                pygame.draw.line (surf, (100, 100, 100), surf_coord (serv[0]), surf_coord (self.servers[neighbour][0]))
        for serv in self.servers:
            pygame.draw.circle (surf, player_colors[serv[2]], surf_coord (serv[0]), 3)
        return surf
