reserved_words = ["IF", "PASSIVE", "AGGRESSIVE", "ATTACK", "DEFEND", "WAIT", "GOTO"]

def check_label (line, index):
    sp = line.split ()
    if len (sp) != 1:
        raise RuntimeError ("La ligne " + str (index + 1) + " est incorrecte ! Le nom d'un état ne peut pas contenir d'espaces.")
    if sp[0] in reserved_words:
        raise RuntimeError ("La ligne " + str (index + 1) + " est incorrecte ! " + sp[0] + " est un nom reservé et ne peut pas être attribué à un état.")
    return sp[0]

def check_behaviour (line, index):
    sp = line.split ()
    error_text = "La ligne " + str (index + 1) + " devrait définir une action pour l'état en cours, parmi ATTACK, WAIT, et DEFEND."
    if len (sp) != 1:
        raise RuntimeError (error_text)
    if sp[0] == "ATTACK":
        return 0
    elif sp[0] == "WAIT":
        return 1
    elif sp[0] == "DEFEND":
        return 2
    else:
        raise RuntimeError (error_text)

def check_case (line, aggressive, passive, index):
    error = "La ligne " + str (index + 1) + " est incorrecte ! Elle devrait être de la forme IF [comportement] GOTO [nom d'état], où comportement est soit PASSIVE soit AGGRESSIVE."
    sp = line.split()
    if len (sp) != 4:
        raise RuntimeError (error)
    if sp[0] != "IF" or sp[2] != "GOTO":
        raise RuntimeError (error)
    if sp[1] == "PASSIVE":
        return aggressive, sp[3]
    elif sp[1] == "AGGRESSIVE":
        return sp[3], passive
    else:
        raise RuntimeError (error)

def check_automaton (automaton):
    if len (automaton) == 0:
        raise RuntimeError ("Le programme est vide !")
    if not ("START" in automaton):
        raise RuntimeError ("Le programme doit avoir un état appelé START, qui sert de point de départ.")
    for state in automaton:
        if not (automaton[state][1] in automaton):
            raise RuntimeError ("L'état " + state + " fait référence à l'état " + automaton[state][1] + ", qui n'existe pas dans ce programme.")
        if not (automaton[state][2] in automaton):
            raise RuntimeError ("L'état " + state + " fait référence à l'état " + automaton[state][2] + ", qui n'existe pas dans ce programme.")
    return automaton


def parse (lines):
    automaton = {}
    index = 0

    while index < len (lines):
        while lines[index].split () == []:
            index += 1
            if index >= len (lines):
                break
        if index < len(lines):
            label = check_label (lines[index], index)
            if label in automaton:
                raise RuntimeError ("L'état déclaré à la ligne " + str (index + 1) + " utilise un nom qui a déjà été défini pour un autre état.")
            if index + 3 >= len (lines):
                raise RuntimeError ("La déclaration de l'état qui commence à la ligne " + str (index + 1) + " est incomplète ! Elle devrait contenir au moins une action (ATTACK, DEFEND ou WAIT), et deux réactions de la forme IF [comportement] GOTO [nom d'état].")
            behaviour = check_behaviour (lines[index + 1], index + 1)
            aggressive_case, passive_case = "", ""
            aggressive_case, passive_case = check_case (lines[index + 2], aggressive_case, passive_case, index + 2)
            aggressive_case, passive_case = check_case (lines[index + 3], aggressive_case, passive_case, index + 3)
            if passive_case == "" or aggressive_case == "":
                raise RuntimeError ("Dans la déclaration de l'état qui commence à la ligne " + str (index + 1) + ", les deux réactions font suite au même comportement. Il doit y avoir une réaction à AGGRESSIVE et une réaction à PASSIVE.")
            automaton[label] = (behaviour, aggressive_case, passive_case)
            index += 4
    return check_automaton (automaton)
